#!/usr/bin/env bash

function showHelp(){
    echo '                                                                                         '
    echo '                                                                                         '
    echo '  Данный скрипт при запуске без параметров устанавливает и настраивает на                '
    echo '  предварительно подготовленном сервере(см README.MD) службу брокера сессий x2go         '
    echo '  Может быть запущен со следующими параметрами:                                          '
    echo '                                                                                         '
    echo '                                                                                         '
    echo '                                                                                         '
    echo '         -h | Выводит эту справочную информацию и завершает работу                       '
    echo '                                                                                         '
    echo '         -i | Непосредственно устанавливает пакеты x2gobroker                            '
    echo '                                                                                         '
    echo '         -c | Редактирует конфигурационные файлы в соответствии с шаблонами из           '
    echo '              комплекта                                                                  '
    echo '                                                                                         '
    echo '         -d | Отключает конфликтующие службы, после чего запускает и добавляет в         '
    echo '              автозагрузку службы брокера                                                '
    echo '                                                                                         '
    echo '         -r | Перезапускает службы брокера                                               '
    echo '                                                                                         '
    echo '         -p | устанавливает модули Python подходящих версий                              '
    echo '                                                                                         '
    echo '         -s | настраивает доступ по SSH для пользователя брокера                         '
    echo '                                                                                         '
    echo '         -b | Исправляет права доступа к исполняемым файлам брокера                      '
    echo '                                                                                         '
    echo '         -k | Генерирует ключи брокера и публикует их для терминальных серверов          '
    echo '                                                                                         '
    echo '     При попытке запуска с неизвестным параметром выйдет с кодом 25                      '
    echo '                                                                                         '
    echo '                                                                                         '
    echo ''
}

set -eu

if [[ $(id -u) -ne 0 ]] ; then
    echo "Please run as root"
    exit 1
fi

#scriptpathmc=$(pwd)
#cd ..
#RootPathMC=$(pwd)
#Надо тестануть этот кусок

#if  ls ./broker_installer.sh 2>/dev/null 1>/dev/null ; then
#    scriptpathmc=$(pwd)
#    cd ..
#    RootPathMC=$(pwd)
#else
#    scriptpathmc=$(find . -name "broker_installer.sh" -execdir pwd {} \; 2>/dev/null)
#    cd "${scriptpathmc}"
#    cd ../..
#    RootPathMC=$(pwd)
#fi

RootPathMC=/usr/share
ConfPathMC=${RootPathMC}'/altconfigs/broker'
SettingsPathMC="/usr/share/x2go_installer/conf"

#Installing x2go broker
function installBroker() {
    apt-get update
    apt-get -y dist-upgrade

    apt-get -y install x2gobroker x2gobroker-common x2gobroker-ssh x2gobroker-daemon x2gobroker-authservice x2gobroker-loadchecker x2gobroker-agent

    echo '***Завершено*** Пакеты сервера терминалов x2gobroker устанавлены'
}

#adding configs
function addConfigs() {
 #   IFS=$'\n'
    while IFS= read -r filename
 #   for  in $(cat ${1})
    do
        DestFile="/${filename}"
        SrcFile="${ConfPathMC}/${filename}"

        if cmp -s "${DestFile}" "${SrcFile}" ; then
            echo "${DestFile} ${SrcFile}"
            echo ' no changes need '
        else
            echo " Replacing ${DestFile} by ${SrcFile} "
            mv -f "${DestFile}" "${DestFile}_$(date +%y%m%d%H).bak"
            cat "${SrcFile}" >  "${DestFile}"
        fi

    done < <(grep -v '^ *#' < "${1}")
    chown -R root:root /etc/x2go/
    
    hostssp=$(cat "${SettingsPathMC}/hostsforsp")

    #     Поиск раздела [pool-C-XFCE]   #Поиск раздела host      #Замена на содержимое файла hostsforsp  <== расшифровка регулярки
    sed -i -e '/^\[pool-C-XFCE\]/,/^\[.*\]/s/^\(\host[ \t]*=[ \t]*\).*$/\'"${hostssp}"'/' /etc/x2go/broker/x2gobroker-sessionprofiles.conf

    echo '***Завершено*** Завершено конфигурирование службы x2gobroker '

}


#setting up services
function setupServices() {
    #ahttpd почему-то нет на тестовых
    systemctl disable avahi-daemon alteratord #ahttpd
    systemctl stop avahi-daemon alteratord #ahttpd

    systemctl enable x2gobroker-authservice x2gobroker-daemon x2gobroker-loadchecker
    systemctl start x2gobroker-authservice x2gobroker-daemon x2gobroker-loadchecker

    echo '***Завершено*** Службы x2goserver успешно запущены '

}


#restarting services
function restartBrokerServices() {

    systemctl restart x2gobroker-authservice x2gobroker-daemon x2gobroker-loadchecker

    echo '***Завершено*** Службы x2goserver успешно перезапущены '

}

#setting up python
function setupPython() {
    apt-get -y install python3-module-pip
    pip3 install --upgrade pip
    pip install --force-reinstall cryptography==2.4.2
    pip install --force-reinstall paramiko==2.6.0
    pip install docutils
    pip install --force-reinstall idna==2.9
    pip check

    echo '***Завершено*** Настройка пакета Python успешно завершена '

    }



#Configurring ssh and sudo

#...
function setupSSH() {
    
    mkdir -p /var/lib/x2gobroker/.ssh/
    
    cp -f /root/.ssh/known_hosts  /var/lib/x2gobroker/.ssh/known_hosts
    #cat /root/.ssh/known_hosts > /var/lib/x2gobroker/.ssh/known_hosts

    chown x2gobroker:x2gobroker /var/lib/x2gobroker/.ssh/
    chown x2gobroker:x2gobroker /var/lib/x2gobroker/.ssh/known_hosts
    
    sed -i '/x2gobroker/d' /etc/openssh/sshd_config
    sed -i '/x2gobroker-users/d'  /etc/openssh/sshd_config
    sed -i -e 's/AllowGroups/AllowGroups x2gobroker x2gobroker-users /' /etc/openssh/sshd_config
    systemctl reload sshd && systemctl restart sshd
    restartBrokerServices

    echo '***Завершено*** Настройка службы ssh успешно завершена '

   }

   
#Tweakind alt linux version of x2gobroker
function tweakFMod() {

    chown x2gobroker:x2gobroker-users /usr/bin/x2gobroker-ssh
    chmod 4550 /usr/bin/x2gobroker-ssh
    chmod g+r /usr/lib/x2go/x2gobroker-agent
    restartBrokerServices

    echo '***Завершено*** Конфигурация файлов x2gobroker успешно завершена '

}

function setupKeys() {

    x2gobroker-keygen
    restartBrokerServices
    x2gobroker-pubkeyauthorizer --broker-url http://localhost:8080/pubkeys
    
    #for all users who need terminal access
    usermod -aG x2gobroker,x2gobroker-users admin
    usermod -s /bin/bash x2gobroker
    restartBrokerServices

    echo '***Завершено*** Настройка ключей завершена '

}


if [[ "$#" -eq 0 ]]
then   

  #Installing x2go broker
    installBroker
  #adding configs
    FilesList="${SettingsPathMC}/broker_configs_list"
    addConfigs "${FilesList}"
  #setting up services
    setupServices
  #setting up python
    setupPython
    restartBrokerServices
  #some settings to ssh
    setupSSH
    restartBrokerServices
  #Twickind alt linux version of x2gobroker
    tweakFMod
  #Tunning x2gobroker
    setupKeys
  #don't shure. looks like works in default
  usermod -s /bin/bash x2gobroker
    echo "Завершено!"
    exit 0
fi

while [ -n "${1}" ]
do
    case "${1}" in
        -h) showHelp
        break   ;;
        -i) installBroker
        break   ;;
        -c) FilesList="${SettingsPathMC}/broker_configs_list"
            addConfigs "${FilesList}"
        break   ;;
        -d) setupServices
        break   ;;
        -p) setupPython
            restartBrokerServices
        break   ;;
        -s) setupSSH
            restartBrokerServices
        break   ;;
        -b) tweakFMod
        break   ;;
        -k) setupKeys
        break   ;;
        -r) restartBrokerServices
        break   ;;        
        *) echo "Неизвестный параметр. Выход."
            exit 25 ;;
    esac
    shift
done
