#!/usr/bin/env bash

function showHelp(){
    echo '                                                                                         '
    echo '                                                                                         '
    echo '  Данный скрипт при запуске без параметров устанавливает и настраивает на                '
    echo '  предварительно подготовленном сервере(см README.MD) службу брокера сессий x2go         '
    echo '  Может быть запущен со следующими параметрами:                                          '
    echo '                                                                                         '
    echo '                                                                                         '
    echo '                                                                                         '
    echo '         -h | Выводит эту справочную информацию и завершает работу                       '
    echo '                                                                                         '
    echo '         -i | Непосредственно устанавливает пакеты x2goserver                            '
    echo '                                                                                         '
    echo '         -c | Редактирует конфигурационные файлы в соответствии с шаблонами из           '
    echo '              комплекта                                                                  '
    echo '                                                                                         '
    echo '         -d | Отключает конфликтующие службы, после чего запускает и добавляет в         '
    echo '              автозагрузку службы сервера                                                '
    echo '                                                                                         '
    echo '         -r | Перезапускает службы брокера                                               '
    echo '                                                                                         '
    echo '         -p | устанавливает модули Python подходящих версий                              '
    echo '                                                                                         '
    echo '         -s | настраивает доступ по SSH для пользователя брокера                         '
    echo '                                                                                         '
    echo '         -b | Исправляет права доступа к исполняемым файлам брокера                      '
    echo '                                                                                         '
    echo '         -k | Генерирует ключи брокера и публикует их для терминальных серверов          '
    echo '                                                                                         '
    echo '     При попытке запуска с неизвестным параметром выйдет с кодом 25                      '
    echo '                                                                                         '
    echo '                                                                                         '
    echo ''
}

set -eu

if [[ $(id -u) -ne 0 ]] ; then 
    echo "Please run as root" 
    exit 1 
fi

#if ls ./terminal_installer.sh  2>/dev/null 1>/dev/null ; then
#    scriptpathmc=$(pwd)
#    cd ..
#    RootPathMC=$(pwd)
#else
#    RootPathMC=$(find . -type d -name "scripts" -execdir pwd {} \; 2>/dev/null)
#    scriptpathmc="${RootPathMC}/scripts/shell" #`find . -type d -name "scripts" -execdir pwd {} \;`
#    #cd $scriptpathmc
#    #cd ..
#    #RootPathMC=$(pwd)
#fi

RootPathMC=/usr/share
ConfPathMC=${RootPathMC}'/altconfigs/terminal/'
SettingsPathMC="/usr/share/x2go_installer/conf"

#Installing x2go server
function installTerminal() {
    apt-get update
    apt-get -y dist-upgrade

    apt-get -y install xfce4-full

    apt-get -y install x2goserver x2goserver-printing cups-x2go x2gobroker-agent x2gobroker-common 

    echo '***Завершено*** Пакеты сервера терминалов x2goserver устанавлены'

}

#adding configs
function addConfigs() {
    #IFS=$'\n'
    #for FileName in `cat $1`
    while IFS= read -r FileName
    do
        DestFile='/'${FileName}
        SrcFile="${ConfPathMC}${FileName}"

        if cmp -s "${DestFile}" "${SrcFile}" ; then
            echo "${DestFile} ${SrcFile}"
            echo ' no changes need '
        else
            echo " Replacing ${DestFile} by ${SrcFile} "
            mv -f "${DestFile}" "${DestFile}_$(date +%y%m%d%H).bak"
            cat "${SrcFile}" >  "${DestFile}"
        fi

    done < <(grep -v '^ *#' < "${1}")
    chown -R root:root /etc/x2go/

    echo '***Завершено*** Завершено конфигурирование службы x2goserver '

}

#setting up services
function setupServices() {
    #ahttpd почему-то нет на тестовых
    systemctl disable avahi-daemon alteratord #ahttpd
    systemctl stop avahi-daemon alteratord #ahttpd

    systemctl enable x2goserver
    systemctl start x2goserver

    echo '***Завершено*** Службы x2goserver успешно запущены '

}

#restarting services
function restartTerminalServices() {

    systemctl restart x2goserver

    echo '***Завершено*** Службы x2goserver успешно перезапущены '

}

#setting up python

function setupPython() {
    apt-get -y install python3-module-pip
    pip3 install --upgrade pip
    pip install --force-reinstall cryptography==2.4.2
    pip install --force-reinstall paramiko==2.6.0
    #pip install --force-reinstall paramiko==2.0.0
    #pip install --force-reinstall cryptography==1.7.1
    pip install docutils
    pip install --force-reinstall idna==2.9
    pip check

    echo '***Завершено*** Настройка пакета Python успешно завершена '

}


#Configurring ssh and sudo 
function setupSSH() {
#...
#Перенесено в system_configurator.sh
#ssh -o HostKeyAlgorithms=ssh-rsa root@termserv
#ssh -o HostKeyAlgorithms=ssh-rsa root@broker
    mkdir -p /var/lib/x2gobroker/.ssh/
    cat /root/.ssh/known_hosts > /var/lib/x2gobroker/.ssh/known_hosts
    chown x2gobroker:x2gobroker /var/lib/x2gobroker/.ssh/
    chown x2gobroker:x2gobroker /var/lib/x2gobroker/.ssh/known_hosts
    sed -i '/x2gobroker/d' /etc/openssh/sshd_config
    sed -i '/x2gobroker-users/d'  /etc/openssh/sshd_config
    sed -i -e 's/AllowGroups/AllowGroups x2gobroker x2gobroker-users/' /etc/openssh/sshd_config
    systemctl reload sshd && systemctl restart sshd
    restartTerminalServices

    echo '***Завершено*** Настройка службы ssh успешно завершена '

}

#Tweakind alt linux version of x2gobroker
function tweakFMod() {
 

    #chown x2gobroker:x2gobroker-users /usr/bin/x2gobroker-ssh
    #chmod 4550 /usr/bin/x2gobroker-ssh
    chmod g+r /usr/lib/x2go/x2gobroker-agent
    restartTerminalServices
    #Tunning x2gobroker

    echo '***Завершено*** Конфигурация файлов x2gobroker-agent успешно завершена '

}

function setupKeys() {

    BrokerName=$(awk '{print $2}' "${SettingsPathMC}/brokerhostname")
    x2gobroker-pubkeyauthorizer --broker-url http://"${BrokerName}":8080/pubkeys
    restartTerminalServices
    #don't shure. looks like works in default
    usermod -s /bin/bash x2gobroker

    #for all users who need terminal access
    usermod -aG x2gobroker,x2gobroker-users admin

    echo '***Завершено*** Настройка ключей завершена '

}

if [[ "$#" -eq 0 ]]
then 

#Installing x2go server

installTerminal

#adding configs

FilesList="${SettingsPathMC}/terminal_configs_list"

addConfigs "${FilesList}"

#setting up services

setupServices

#setting up python

setupPython

restartTerminalServices

setupSSH

tweakFMod

setupKeys  

    echo "Завершено!"
    exit 0
fi

while [ -n "$1" ]
do
    case "$1" in
        -h) showHelp
        break   ;;
        -i) installTerminal
        break   ;;
        -c) FilesList="${SettingsPathMC}/terminal_configs_list"
            addConfigs "${FilesList}"
        break   ;;
        -d) setupServices
        break   ;;
        -p) setupPython
            restartTerminalServices
        break   ;;
        -s) setupSSH
            restartTerminalServices
        break   ;;
        -b) tweakFMod
        break   ;;
        -k) setupKeys 
        break   ;;
        -r) restartTerminalServices        
        break   ;;
        *) echo "Неизвестный параметр. Выход."
            exit 25 ;;
    esac
    shift
done