#!/usr/bin/env bash

function showHelp(){
    echo '                                                                                         '
    echo '                                                                                         '
    echo '  Данный скрипт выполняет настройку SSH и sudo                                           '
    echo '  Может быть запущен со следующими параметрами:                                          '
    echo '                                                                                         '
    echo '                                                                                         '
    echo '                                                                                         '
    echo '         -h | Выводит эту справочную информацию и завершает работу                       '
    echo '                                                                                         '
    echo '         -i | Применяет настройки ssh в конфиге sshd_config, после чего перезапускает    '
    echo '              службу sshd                                                                '
    echo '                                                                                         '
    echo '         -s | Применяет настройки в /etc/sudoers с целью разрешения использования        '
    echo '              команды sudo пользователям в группах wheel и sudo                          '
    echo '                                                                                         '
    echo '         -a | Добавляет хосты из /root/hosts в список известных хостов для ssh           '
    echo '              выполнять после запука этого скрипта с параметрами -i и -s на каждом       '
    echo '              сервере из списка                                                          '
    echo '                                                                                         '
    echo '         -r | Удаляет установленные сервисы x2go и удаляет связанные конфигурационные    '
    echo '              файлы                                                                      '
    echo '                                                                                         '
    echo '     При попытке запуска с неизвестным параметром выйдет с кодом 25                      '
    echo '                                                                                         '
    echo '                                                                                         '
}

set -eu

if [[ $(id -u) -ne 0 ]] ; then
    echo "Please run as root"
    exit 1
fi


#Определитель папки, пока не понадобился
#if [[ $(ls ./system_configurator.py  2>/dev/null 1>/dev/null) -eq 0 ]] ; then
#    scriptpathmc=$(pwd)
#    cd ..
#    rootpathmc=$(pwd)
#else    
#    scriptpathmc=$(find . -type d -name "scripts" -execdir pwd {} \; 2>/dev/null)
#    cd "${scriptpathmc}"
#    cd ..
#    rootpathmc=$(pwd)
#fi

function setupSSH(){
    sed -i.bak -e 's/#PermitRootLogin without-password/PermitRootLogin yes/' /etc/openssh/sshd_config
    sed -i -e 's/#PubkeyAuthentication yes/PubkeyAuthentication yes/' /etc/openssh/sshd_config
    sed -i -e 's/#AllowGroups/AllowGroups root /' /etc/openssh/sshd_config
    sed -i -e 's/#KillUserProcesses=yes/KillUserProcesses=no /' /etc/systemd/logind.conf

# Надо запускать после того как установлен x2go
# sed -i -e 's/AllowGroups/AllowGroups x2gobroker x2gobroker-users /' /etc/sshd.conf
    systemctl reload sshd && systemctl restart sshd
    echo "Настройка ssh выполнена, служба перезапущена."
}

function setupSudo(){
    sed -i.bak -e 's/# root ALL=(ALL) ALL/root ALL=(ALL) ALL/' /etc/sudoers
    sed -i -e 's/# User_Alias    SUDO_USERS = %sudo/User_Alias    SUDO_USERS = %sudo/' /etc/sudoers
    sed -i -e 's/# WHEEL_USERS ALL=(ALL) ALL/WHEEL_USERS ALL=(ALL) ALL/' /etc/sudoers
    sed -i -e 's/# SUDO_USERS/SUDO_USERS/' /etc/sudoers

    echo "Настройка sudo выполнена, требуется выполнить выход из системы для применения настроек."
}

function addKnownHosts(){
    #cat ${rootpathmc}/altconfigs/hosts >> /etc/hosts
    # Реализовано в hosts_maker.py
    #cat /root/hosts >> /etc/hosts
    for FarmHost in $(awk '{print $2}' /root/hosts)
    do
        ssh -o HostKeyAlgorithms=ssh-rsa root@"${FarmHost}" -n /bin/true
    done 

    echo "Добавление серверов в список известных хостов ssh выполнено."
}

function cleanSystem(){
    
    apt-get remove -y x2go*

    rm -fr /etc/x2go
    rm -fr /var/lib/x2gobroker
    rm -fr /var/log/x2gobroker
    rm -fr /usr/lib/x2go
    rm -fr /usr/share/x2go

    echo "Удаление сервисов x2go выполнено."
}

if [[ "$#" -eq 0 ]]
then    
    echo "Параметры не заданы!"
    echo "Выполняется настройка SSH и SUDO."
    echo "После выполнения на всех серверах необходимо запустить этот скрипт с параметром -a"
    
    setupSSH
    setupSudo
    exit 0
fi

while [ -n "$1" ]
do
    case "$1" in
        -h) showHelp
        break   ;;
        -i) setupSSH
        break   ;;
        -s) setupSudo
        break   ;;
        -a) addKnownHosts
        break   ;;
        -r) cleanSystem
        break   ;;
        *) echo "Неизвестный параметр. Выход."
        exit 25 ;;
    esac
    shift
done