#!/usr/bin/env bash
# https://www.8host.com/blog/migraciya-virtualnogo-servera-linux-peremeshhenie-polzovatelej-grupp-pochty-i-drugix-nastroek/
function show_help(){
    echo '                                                                                         '
    echo '                                                                                         '
    echo '  Данный скрипт переносит локальных пользователей с целевого сервера на текущий          '
    echo '  Использование:                                                                         '
    echo '  user_migrator.sh SERVER -g -s [-b | -r]                                                '
    echo '                                                                                         '
    echo '         -h | Выводит эту справочную информацию и завершает работу                       '
    echo '                                                                                         '
    echo '         -g | Получение данных с удаленного сервера (SERVER) должен быть настроен        '
    echo '              доступ по ssh по ключу для пользователя root                               '
    echo '                                                                                         '
    echo '         -s | Импорт полученных данных на текущий сервер                                 '
    echo '                                                                                         '
    echo '         -b | Создание резервной копии пользователей на локальном сервере                '
    echo '                                                                                         '
    echo '         -r | Восстановление данных из ранее создвнной копии                             '
    echo '                                                                                         '
    echo '     При попытке запуска с неизвестным параметром выйдет с кодом 25                      '
    echo '                                                                                         '
    echo '                                                                                         '
    echo ''
}

set -eu

WorkDir=/usr/share/x2go_installer/migr
TempDir=/usr/share/x2go_installer/temp
#mkdir /root/fmigr
Limit=500

UsersFile="/etc/passwd"
GroupsFile="/etc/group"
UsersShFile="/etc/shadow"
GroupsShFile="/etc/gshadow"
TcbDir="/etc/tcb/"

function initialize () {

    Server="${1}"
    
    UsersSyncFile=/passwd."${Server}".sync
    UsersSyncFileMod="${UsersSyncFile}"mod

    GroupsSyncFile=/group."${Server}".sync
    GroupsSyncFileFull="${GroupsSyncFile}"full

    if [[ $(ls "${WorkDir}"  2>/dev/null 1>/dev/null) -gt 0 ]] ; then
        mkdir "${WorkDir}"
    fi

    if [[ $(ls "${TempDir}"  2>/dev/null 1>/dev/null) -gt 0 ]] ; then
        mkdir "${TempDir}"
    fi
}

function checkserver () {

    if [ -z "${Server}" ]
    then    
        echo "Сервер не задан!"
        exit 2
    fi

}
#
function fullBackup () { 

    BackupFile="${WorkDir}/users_backup.tar"

    if [[ $(ls "${WorkDir}"  2>/dev/null 1>/dev/null) -gt 0 ]] ; then
        UserBackupFile="users_backup"
        UserBackupFile="${UserBackupFile}"$(stat -c %y%m%d%H  x2ggof.tar | awk '{print $1}' | awk -F "-" '{print $1$2$2}')
        UserBackupFile="${UserBackupFile}"$(stat -c %y%m%d%H  x2ggof.tar | awk '{print $2}' | awk -F ":" '{print $1$2}')
        UserBackupFile="${UserBackupFile}"".tar"

        mv "${BackupFile}" "${WorkDir}${UserBackupFile}"
    fi

    tar -cvf "${BackupFile}" "${UsersFile}"
    tar -rvf "${BackupFile}" "${GroupsFile}"
    tar -rvf "${BackupFile}" "${UsersShFile}"
    tar -rvf "${BackupFile}" "${GroupsShFile}"
    tar -rvf "${BackupFile}" "${TcbDir}"

    echo "Процесс создания резервной копии завершен"

}

function fullRestore () { 
    
    BackupFile="${WorkDir}/users_backup.tar"
    
    tar -xvf "${BackupFile}" -C "/"
    
    echo "Процесс восстановления из резервной копии завершен"

}

function getUserData () {
    
    ssh "${Server}"  cat ${UsersFile} | awk -v LIMIT="${Limit}" -F: '($3>=LIMIT) && ($3!=65534)'  > "${WorkDir}${UsersSyncFile}"
    ssh "${Server}"  cat ${GroupsFile} | awk -v LIMIT="${Limit}" -F: '($3>=LIMIT) && ($3!=65534)'  > "${WorkDir}${GroupsSyncFile}"
    scp "${Server}":${GroupsFile} "${WorkDir}${GroupsSyncFileFull}"
    for Nuser in $(awk -F: '{print $1}' "${WorkDir}${UsersSyncFile}")
    do
        ssh "${Server}" tar zc "${TcbDir}""${Nuser}" | tar zx -C "${WorkDir}"
    done

}

function setUserData () {
    
    while IFS= read -r Line
    do
        Nuser=$(echo "${Line}" | awk -F: '{print $1}')
        if id "${Nuser}"  2>/dev/null 1>/dev/null ; then
            sed -i '/'"${Nuser}"':.*/d' "${WorkDir}${UsersSyncFile}"
        fi
    done < <(grep -v '^ *#' < "${WorkDir}${UsersSyncFile}")

    # Подготовка файла для пакетного добавления пользователей
    awk 'BEGIN { OFS=FS=":"; } {$3=""; $4=""; } { print; }' "${WorkDir}${UsersSyncFile}" > "${WorkDir}${UsersSyncFileMod}"

    # Пакетное добавление пользователей
    newusers "${WorkDir}${UsersSyncFileMod}"

    awk -F: '{print $1}' "${WorkDir}${UsersSyncFile}"
    # Добавление групп новым пользователям
    while IFS= read -r Line
    do
        Nuser=$(echo "${Line}" | awk -F: '{print $1}')
        GroupsUM=$(grep -E '[:|,]'"${Nuser}"'' "${WorkDir}${GroupsSyncFileFull}"| awk -F: '{printf $1","}' | sed 's/,$//g')
        usermod -aG "${GroupsUM}" "${Nuser}"
        cp -ru "${WorkDir}${TcbDir}${Nuser}" "${TcbDir}"
        chown -R "${Nuser}:auth" "${TcbDir}${Nuser}"
        
    done < <(grep -v '^ *#' < "${WorkDir}${UsersSyncFile}")    

}

#Запуск без параметров

if [ -z "$1" ]
then    
    echo "Завершено!"
    exit 1
fi

Server="$1"     
initialize "${Server}"
shift

if [[ $(id -u) -ne 0 ]] ; then
    echo "Please run as root"
    exit 1
fi
Server="$1"        
initialize "${Server}"
shift

while [ -n "$1" ]
do
    case "$1" in
        -h) show_help
        break   ;;
        -b) fullBackup
        break   ;;
        -r) fullRestore
        break   ;;
        -g) getUserData
        break   ;;
        -s) setUserData
        break   ;;
        *) echo "Неизвестный параметр. Выход."
            exit 25 ;;
    esac
    shift
done
